/***************************************************************************
 *   Copyright (C) 2003 by Sbastien Laot                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BASKET_H
#define BASKET_H

#include <qscrollview.h>
#include <qtooltip.h>
#include <qvaluelist.h>
#include <qtimer.h>
#include <qimage.h>
#include <qdatetime.h>
#include <qclipboard.h>
#include <kshortcut.h>
#include <kaction.h>
#include <kio/job.h>

#include "search.h"
#include "item.h" // For Item::Zone

class QVBoxLayout;
class QDomDocument;
class QDomElement;

class Basket;
class Item;
class ItemEditor;
class Tag;


/** This represent a hierarchy of the selected classes.
  * If this is null, then there is no selected item.
  */
class ItemSelection
{
  public:
	ItemSelection()        : item(0), parent(0), firstChild(0), next(0), fullPath() {}
	ItemSelection(Item *i) : item(i), parent(0), firstChild(0), next(0), fullPath() {}

	Item          *item;
	ItemSelection *parent;
	ItemSelection *firstChild;
	ItemSelection *next;
	QString        fullPath; // Needeed for 'Cut' code to store temporary path of the cutted item.

	ItemSelection* firstStacked();
	ItemSelection* nextStacked();
	void append(ItemSelection *node);
	int count();

	QValueList<Item*> parentGroups();
};

/** This store all needed information when exporting to HTML
  */
class HtmlExportData
{
  public:
	QString iconsFolderPath;
	QString iconsFolderName;
	QString imagesFolderPath;
	QString imagesFolderName;
	QString dataFolderPath;
	QString dataFolderName;
	bool    formatForImpression;
	bool    embedLinkedFiles;
	bool    embedLinkedFolders;
};

/** This class handle Basket and add a SearchWidget on top of it.
  * @author Sbastien Laot
  */
class DecoratedBasket : public QWidget
{
  Q_OBJECT
  public:
	DecoratedBasket(QWidget *parent, const QString &folderName, const char *name = 0, WFlags fl = 0);
	~DecoratedBasket();
	void setSearchBarPosition(bool onTop);
	void resetSearch();
	void setSearchBarShown(bool show, bool switchFocus = true);
	bool isSearchBarShown()        { return m_search->isShown();    }
	const SearchData& searchData() { return m_search->searchData(); }
	SearchBar* searchBar()         { return m_search;               }
	Basket*    basket()            { return m_basket;               }
  private:
	QVBoxLayout *m_layout;
	SearchBar   *m_search;
	Basket      *m_basket;
};

/**
  * @author Sbastien Laot
  */
class Basket : public QScrollView, public QToolTip
{
/// CONSTRUCTOR AND DESTRUCTOR:
  Q_OBJECT
  public:
	Basket(QWidget *parent, const QString &folderName);
	~Basket();

/// USER INTERACTION:
  private:
	bool   m_noActionOnMouseRelease;
//	bool   m_mousePressed;
	QPoint m_pressPos;
	bool   m_canDrag;
  public:
	void viewportResizeEvent(QResizeEvent *);
	void drawContents(QPainter *painter, int clipX, int clipY, int clipWidth, int clipHeight);
	void enterEvent(QEvent *);
	void leaveEvent(QEvent *);
	void contentsMouseMoveEvent(QMouseEvent *event);
	void contentsMousePressEvent(QMouseEvent *event);
	void contentsMouseReleaseEvent(QMouseEvent *event);
	void contentsMouseDoubleClickEvent(QMouseEvent *event);
	void updateItem(Item *item);
	void clickedToInsert(QMouseEvent *event, Item *clicked = 0, int zone = 0);

/// LAYOUT:
  private:
	Item   *m_firstItem;
	int     m_columnsCount;
	bool    m_mindMap;
	Item   *m_resizingItem;
	int     m_pickedResizer;
	Item   *m_movingItem;
	QPoint  m_pickedHandle;
  public:
	int tmpWidth;
	int tmpHeight;
  public:
	void unsetItemsWidth();
	void relayoutItems(bool animate);
	Item* itemAt(int x, int y);
	inline Item* firstItem()       { return m_firstItem;                 }
	inline int   columnsCount()    { return m_columnsCount;              }
	inline bool  isColumnsLayout() { return m_columnsCount > 0;          }
	inline bool  isFreeLayout()    { return m_columnsCount <= 0;         }
	inline bool  isMindMap()       { return isFreeLayout() && m_mindMap; }
	Item* resizingItem()           { return m_resizingItem;              }
	Item* lastItem();
	void setDisposition(int disposition, int columnCount);
	void equalizeColumnSizes();

/// ITEM INSERTION AND REMOVAL:
  public:
	/// The following methods assume that the item(s) to insert already all have 'this' as the parent basket:
	void prependItemIn(   Item *item, Item *in);     /// << Add @p item (and the next linked items) as the first item(s) of the group @p in.
	void appendItemIn(    Item *item, Item *in);     /// << Add @p item (and the next linked items) as the last item(s) of the group @p in.
	void appendItemAfter( Item *item, Item *after, bool alreadyInBasket = false);  /// << Add @p item (and the next linked items) just after (just below) the item @p after.
	void appendItemBefore(Item *item, Item *before); /// << Add @p item (and the next linked items) just before (just above) the item @p before.
	void groupItemAfter(  Item *item, Item *with);   /// << Add a group at @p with place, move @p with in it, and add @p item (and the next linked items) just after the group.
	void groupItemBefore( Item *item, Item *with);   /// << Add a group at @p with place, move @p with in it, and add @p item (and the next linked items) just before the group.
	void unplugItem(      Item *item, bool willBeReplugged = false); /// << Unplug @p item (and its child items) from the basket (and also decrease counts...).
	                                                 /// <<  After that, you should delete the items yourself. Do not call prepend/append/group... functions two times: unplug and ok
	void ungroupItem(     Item *group); /// << Unplug @p group but put child items at itsplace.
	/// And this one do almost all the above methods depending on the context:
	void insertItem(Item *item, Item *clicked, int zone, const QPoint &pos = QPoint(), bool animateNewPosition = false);
	void insertCreatedItem(Item *item/*, bool edit = false*/);
  private:
	void preparePlug(Item *item);
	void finishPlug();
  private:
	Item  *m_clickedToInsert;
	int    m_zoneToInsert;
	QPoint m_posToInsert;
	Item  *m_savedClickedToInsert;
	int    m_savedZoneToInsert;
	QPoint m_savedPosToInsert;
  public:
	void saveInsertionData();
	void restoreInsertionData();
	void resetInsertionData();
  public slots:
	void insertEmptyItem(int type);
	void insertWizard(int type);
	void insertColor(const QColor &color);
	void pasteItem(QClipboard::Mode mode = QClipboard::Clipboard);
  private slots:
	void hideInsertPopupMenu();
	void timeoutHideInsertPopupMenu();

/// TOOL TIPS:
  protected:
	void maybeTip(const QPoint &pos);

/// ANIMATIONS:
  private:
	QValueList<Item*> m_animatedItems;
	QTimer            m_animationTimer;
	int               m_deltaY;
//	int               m_loadingFrames;
//	QTimer            m_loadingFramesTimer;
	QTime             m_lastFrameTime;
	static const int FRAME_DELAY;
  private slots:
	void animateObjects();
  public slots:
	void animateLoad();
//	void updateLoadingFrame();
  public:
	void addAnimatedItem(Item *item);

/// LOAD AND SAVE:
  private:
	bool m_loaded;
	bool m_loadingLaunched;
  private slots:
	void loadItems(const QDomElement &items, Item *parent);
	void saveItems(QDomDocument &document, QDomElement &element, Item *parent);
  public slots:
	void load();
	void loadProperties(const QDomElement &properties);
	void saveProperties(QDomDocument &document, QDomElement &properties);
	void save();
  public:
	bool isLoaded()        { return m_loaded;          }
	bool loadingLaunched() { return m_loadingLaunched; }

/// BACKGROUND:
  private:
	QColor   m_backgroundColorSetting;
	QString  m_backgroundImageName;
	QPixmap *m_backgroundPixmap;
	QPixmap *m_opaqueBackgroundPixmap;
	QPixmap *m_selectedBackgroundPixmap;
	bool     m_backgroundTiled;
	QColor   m_textColorSetting;
  public:
	inline bool           hasBackgroundImage()     { return m_backgroundPixmap != 0;          }
	inline const QPixmap* backgroundPixmap()       { return m_backgroundPixmap;               }
	inline bool           isTiledBackground()      { return m_backgroundTiled;                }
	inline QString        backgroundImageName()    { return m_backgroundImageName;            }
	inline QColor         backgroundColorSetting() { return m_backgroundColorSetting;         }
	inline QColor         textColorSetting()       { return m_textColorSetting;               }
	QColor backgroundColor();
	QColor textColor();
	void setAppearance(const QString &icon, const QString &name, const QString &backgroundImage, const QColor &backgroundColor, const QColor &textColor);
	void blendBackground(QPainter &painter, const QRect &rect, int xPainter = -1, int yPainter = -1, bool opaque = false, QPixmap *bg = 0);
	void unbufferizeAll();
	void subscribeBackgroundImages();
	void unsubscribeBackgroundImages();

/// KEYBOARD SHORTCUT:
  private:
	KAction *m_action;
	int      m_shortcutAction;
  private slots:
	void activatedShortcut();
  public:
	KShortcut shortcut() { return m_action->shortcut(); }
	int shortcutAction() { return m_shortcutAction;    }
	void setShortcut(KShortcut shortcut, int action);

/// USER INTERACTION:
  private:
	Item  *m_hoveredItem;
	int    m_hoveredZone;
	bool   m_lockedHovering;
	bool   m_underMouse;
	QRect  m_inserterRect;
	bool   m_inserterShown;
	bool   m_inserterSplit;
	bool   m_inserterTop;
	bool   m_inserterGroup;
	void placeInserter(Item *item, int zone);
	void removeInserter();
  public:
//	bool inserterShown() { return m_inserterShown; }
	bool inserterSplit() { return m_inserterSplit; }
	bool inserterGroup() { return m_inserterGroup; }
	public slots:
	void doHoverEffects(Item *item, Item::Zone zone, const QPoint &pos = QPoint(0, 0)); /// @p pos is optionnal and only used to show the link target in the statusbar
	void doHoverEffects(const QPoint &pos);
	void doHoverEffects(); // The same, but using the current cursor position
  public:
	void popupTagsMenu(Item *item);
	void popupEmblemMenu(Item *item, int emblemNumber);
	void addTagToSelectedItems(Tag *tag);
	void removeTagFromSelectedItems(Tag *tag);
	void removeAllTagsFromSelectedItems();
	void addStateToSelectedItems(State *state);
	void changeStateOfSelectedItems(State *state);
	bool selectedItemsHaveTags();
	const QRect& inserterRect()  { return m_inserterRect;  }
	bool         inserterShown() { return m_inserterShown; }
	void drawInserter(QPainter &painter, int xPainter, int yPainter);
	DecoratedBasket* decoration();
  private slots:
	void toggledTagInMenu(int id);
	void toggledStateInMenu(int id);
	void unlockHovering();
	void disableNextClick();
	void recomputeAllStyles();
	void contentsMoved();
  private:
	Item  *m_tagPopupItem;
	Tag   *m_tagPopup;
	QTime  m_lastDisableClick;

/// SELECTION:
  private:
	bool   m_isSelecting;
	bool   m_selectionStarted;
	bool   m_selectionInvert;
	QPoint m_selectionBeginPoint;
	QPoint m_selectionEndPoint;
	QRect  m_selectionRect;
	QTimer m_autoScrollSelectionTimer;
	void stopAutoScrollSelection();
  private slots:
	void doAutoScrollSelection();
  public:
	inline bool isSelecting() { return m_isSelecting; }
	inline const QRect& selectionRect() { return m_selectionRect; }
	void selectItemsIn(const QRect &rect, bool invertSelection);
	void resetWasInLastSelectionRect();
	void selectAll();
	void unselectAll();
	void invertSelection();
	void unselectAllBut(Item *toSelect);
	void invertSelectionOf(Item *toSelect);
	QColor selectionRectInsideColor();
	Item* theSelectedItem();
	ItemSelection* selectedItems();

/// BLANK SPACES DRAWING:
  private:
	QValueList<QRect> m_blankAreas;
	void recomputeBlankRects();

/// COMMUNICATION WITH ITS CONTAINER:
  signals:
	void postMessage(const QString &message);      /// << Post a temporar message in the statusBar.
	void setStatusBarText(const QString &message); /// << Set the permanent statusBar text or reset it if message isEmpty().
	void resetStatusBarText();                     /// << Equivalent to setStatusBarText("").
	void propertiesChanged(Basket *basket);
	void countsChanged(Basket *basket);
  public slots:
	void linkLookChanged();
	void signalCountsChanged();
  private:
	QTimer m_timerCountsChanged;
  private slots:
	void countsChangedTimeOut();

/// ITEMS COUNTING:
  public:
	void addSelectedItem()    { ++m_countSelecteds;   signalCountsChanged(); }
	void removeSelectedItem() { --m_countSelecteds;   signalCountsChanged(); }
	void resetSelectedItem()  { m_countSelecteds = 0; signalCountsChanged(); } // FIXME: Useful ???
	int count()               { return m_count;          }
	int countFounds()         { return m_countFounds;    }
	int countSelecteds()      { return m_countSelecteds; }
  private:
	int m_count;
	int m_countFounds;
	int m_countSelecteds;

/// PROPERTIES:
  public:
	QString basketName() { return m_basketName; }
	QString icon()       { return m_icon;       }
	QString folderName() { return m_folderName; }
	QString fullPath();
	QString fullPathForFileName(const QString &fileName); // Full path of an [existing or not] item in this basket
	static QString fullPathForFolderName(const QString &folderName);
  private:
	QString m_basketName;
	QString m_icon;
	QString m_folderName;

/// ACTIONS ON SELECTED ITEMS FROM THE INTERFACE:
  public slots:
	void itemEdit(Item *item = 0L, bool justAdded = false, const QPoint &clickedPoint = QPoint());
	void itemDelete();
	void itemDeleteWithoutConfirmation(bool deleteFilesToo = true);
	void itemCopy();
	void itemCut();
	void itemOpen(Item *item = 0L);
	void itemOpenWith(Item *item = 0L);
	void itemSaveAs();
	void itemMoveOnTop();
	void itemMoveOnBottom();
	void itemMoveItemUp();
	void itemMoveItemDown();
  public:
	enum CopyMode { CopyToClipboard, CopyToSelection, CutToClipboard };
	void doCopy(CopyMode copyMode);

/// ITEM EDITION:
  private:
	ItemEditor *m_editor;
	bool        m_redirectEditActions;
  public:
	bool isDuringEdit()        { return m_editor;              }
	bool redirectEditActions() { return m_redirectEditActions; }
	bool hasTextInEditor();
	bool hasSelectedTextInEditor();
	bool selectedAllTextInEditor();
  protected slots:
	void selectionChangedInEditor();
  public slots:
	void placeEditor();
	void closeEditor();

/// FILTERING:
  public slots:
	void newSearch(const SearchData &data);
	void searchAgain();

/// DRAG AND DROP:
  private:
	bool m_isDuringDrag;
	QValueList<Item*> m_draggedItems;
  public:
	static void acceptDropEvent(QDropEvent *event, bool preCond = true);
	void dropEvent(QDropEvent *event);
	void contentsDropEvent(QDropEvent *event);
	bool isDuringDrag() { return m_isDuringDrag; }
	QValueList<Item*> draggedItems() { return m_draggedItems; }
  protected:
	void contentsDragEnterEvent(QDragEnterEvent*);
	void contentsDragMoveEvent(QDragMoveEvent *event);
	void contentsDragLeaveEvent(QDragLeaveEvent*);
  public slots:
	void slotCopyingDone(KIO::Job *, const KURL &, const KURL &to, bool, bool);
	void slotCopyingDone2(KIO::Job *job);
  public:
	Item* itemForFullPath(const QString &path);

/// EXPORTATION:
  public:
	static QString cssFontDefinition(const QFont &font, bool onlyFontFamily = false);
	static QString copyIcon(const QString &iconName, int size, const QString &destFolder);
	static QString copyFile(const QString &srcPath, const QString &destFolder, bool createIt = false);
	QValueList<State*> usedStates();
	static QString saveGradientBackground(const QColor &color, const QFont &font, const QString &folder);
  public slots:
	void exportToHTML();

/// REIMPLEMENTED:
  public:
	/** Travel the list to find the next shown item, or the previous if step == -1, or the next after 10 if step == 10... */
	Item* nextShownItemFrom(Item */*item*/, int /*step*/) {return 0;};

	void deleteFiles();


/// FROM OLD ARCHITECTURE **********************

public slots:

	void showFrameInsertTo() {}
	void resetInsertTo() {}

	void  computeInsertPlace(const QPoint &/*cursorPosition*/)    { }
	void  processActionAsYouType(QKeyEvent */*event*/)    { };
	public slots:
		/** Items manipulation */
		void changeItemPlace(Item */*item*/)    { };
		/** Selection of item(s) */
		void selectRange(Item */*start*/, Item */*end*/)    { };
		void clicked(Item */*item*/, bool /*controlPressed*/, bool /*shiftPressed*/)    { };
		void setFocusedItem(Item */*item*/)    { };
		void focusAnItem()    { };
		void ensureVisibleItem(Item */*item*/)    { };
		QRect itemRect(Item */*item*/)    { return QRect();}; // clipped global (desktop as origin) rectangle
	public:
		void    dontCareOfCreation(const QString &/*path*/)    { };
	protected slots:
		void slotModifiedFile(const QString &/*path*/)    { };
		void slotCreatedFile(const QString &/*path*/)    { };
		void slotDeletedFile(const QString &/*path*/)    { };
		void slotUpdateItems()    { };
		void clipboardChanged(bool /*selectionMode*/ = false)    { };
		void selectionChanged()    { };
	public:
		bool importLauncher(const QString &/*type*/, const QDomElement &/*content*/, const QString &/*runCommand*/,
							const QString &/*annotations*//*, bool checked*/)    { return false; }

	inline bool    isLocked()               { return 0;}//m_isLocked;               }
	inline bool    showSearchBar()          { return 0;}//m_showSearchBar;          } // Others... ::
	/** Basket properties SET */
	void setLocked(bool /*lock*/) {};
	void setShowSearchBar(bool /*show*/)         { }

	friend class ContainerSystemTray;
};









#if 0

#include <qwidget.h>
#include <qscrollview.h>
#include <qclipboard.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <kio/job.h>
#include <qcolor.h>

#include "search.h"

class QFrame;
class QVBoxLayout;
class QCheckBox;
class QString;
class QColor;
class QPixmap;
class QAction;
class QStringList;
class QRect;

class QDomElement;

class KDirWatch;

class Basket;
class Item;
class ItemEditorBase;

/** Used to enqueue a file path when the Basket receive a file modification / creation / deletion
  * It associate the file name with an event.
  * All this queue will be treated later.
  * TODO: rename to class WatcherEvent ?
  * @author Sbastien Laot
  */
class FileEvent
{
  public:
	enum Event { Modified = 1, Created, Deleted, Renamed };
	FileEvent(Event evt, const QString &path)
	 : event(evt), filePath(path)
	{ }
  public: // Because it must be fast and theire is no need to be private
	Event   event;
	QString filePath;
};

/** Basket that contain some Items.
  * @author Sbastien Laot
  */
clas   s Bas    ket : public QScrollView
{
  Q_OBJECT
  public:
	/** Construtor and destructor */
	Bask      et(QWidget *parent, const QString &folderName, const char *name = "", WFlags fl = 0);
  public:
  protected:
	virtual void contextMenuEvent(QContextMenuEvent *event);
	virtual void focusInEvent(QFocusEvent*);
	virtual void focusOutEvent(QFocusEvent*);
	virtual void keyPressEvent(QKeyEvent *event);
	virtual void contentsMousePressEvent(QMouseEvent *event); // For redirected event !!
	virtual void showEvent(QShowEvent *);
	/** Drag and drop functions */
	virtual void dragEnterEvent(QDragEnterEvent*);
	virtual void dragMoveEvent(QDragMoveEvent* event);
	virtual void dragLeaveEvent(QDragLeaveEvent*);
  public:
	virtual void dropEvent(QDropEvent *event);
	static  void acceptDropEvent(QDropEvent *event, bool preCond = true);
	bool  canDragItem()       { return !isEmpty(); }
	void  computeInsertPlace(const QPoint &cursorPosition);
	Item* itemAtPosition(const QPoint &pos);
	Item* duplicatedOf(Item *item);
	void  checkClipboard();
	void  processActionAsYouType(QKeyEvent *event);
	void  exportToHTML();
	void  clearDirectory(const QString &path);
  signals:
	void nameChanged(Basket *basket, const QString &name);
	void iconChanged(Basket *basket, const QString &icon);
	void itemsNumberChanged(Basket *basket);
  public slots:
	void linkLookChanged();
	void showItemsToolTipChanged();
	/** Items manipulation */
	void insertItem(Item *item);
	void delItem(Item *item, bool askForMirroredFile = true);
	void changeItemPlace(Item *item);
	void pasteItem(QClipboard::Mode mode = QClipboard::Clipboard);
	void recolorizeItems();
	void reloadMirroredFolder();
	void showMirrorOnlyOnceInfo();
	/** Selection of item(s) */
	void selectAll();
	void unselectAll();
	void unselectAllBut(Item *toSelect);
	void invertSelection();
	void selectRange(Item *start, Item *end);
	void clicked(Item *item, bool controlPressed, bool shiftPressed);
	void setFocusedItem(Item *item);
	void focusAnItem();
	void ensureVisibleItem(Item *item);
	QRect itemRect(Item *item); // clipped global (desktop as origin) rectangle
	/** Travel the list to find the next shown item, or the previous if step == -1, or the next after 10 if step == 10... */
	Item* nextShownItemFrom(Item *item, int step);
	/** Actions on (selected) items */
	void editItem(Item *item, bool editAnnotations = false);
	void editItem();
	void editItemMetaData();
	void delItem();
	void copyItem();
	void cutItem();
	void openItem();
	void openItemWith();
	void saveItemAs();
	void moveOnTop();
	void moveOnBottom();
	void moveItemUp();
	void moveItemDown();
  public:
	void    dontCareOfCreation(const QString &path);
	QString copyIcon(const QString &iconName, int size, const QString &destFolder);
	QString copyFile(const QString &srcPath, const QString &destFolder, bool createIt = false);
  protected slots:
	void slotModifiedFile(const QString &path);
	void slotCreatedFile(const QString &path);
	void slotDeletedFile(const QString &path);
	void slotUpdateItems();
	void placeEditor();
	void closeEditor(bool save = true);
	void clipboardChanged(bool selectionMode = false);
	void selectionChanged();
  private:
	QTimer              m_updateTimer;
	QPtrList<FileEvent> m_updateQueue;
	QStringList         m_dontCare;
	static const int    c_updateTime;
  private:
	void load(); // Load is performed only once, during contructor
	void loadItems(const QDomElement &items);
	bool importLauncher(const QString &type, const QDomElement &content, const QString &runCommand,
	                    const QString &annotations/*, bool checked*/);
	void computeShownItems();

  private:

	Item      *m_focusedItem;
	Item      *m_startOfShiftSelectionItem;

  private:
	KDirWatch      *m_watcher;
	ItemEditorBase *m_editor;
	QKeyEvent      *m_stackedKeyEvent;
};

#endif // #if 0

#endif // BASKET_H
