/***************************************************************************
 *   Copyright (C) 2003 by Sbastien Laot                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <klineedit.h>
#include <kurlrequester.h>
#include <kicondialog.h>
#include <kcolordialog.h>
#include <kservice.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kmainwindow.h>
#include <ktoolbar.h>
#include <kaction.h>
#include <kurifilter.h>

#include "itemedit.h"
#include "itemcontent.h"
#include "itemfactory.h" // Use Tools::
#include "item.h"
#include "basket.h"
#include "settings.h"
#include "utilities.h"
#include "variouswidgets.h"
#include "focusedwidgets.h"

#include <iostream>

/** class ItemEditor: */

ItemEditor::ItemEditor(ItemContent *itemContent)
{
	m_isEmpty  = false;
	m_canceled = false;
	m_widget   = 0;
	m_textEdit = 0;
	m_lineEdit = 0;
	m_itemContent = itemContent;
}

Item* ItemEditor::item()
{
	return m_itemContent->item();
}

ItemEditor* ItemEditor::editItemContent(ItemContent *itemContent, QWidget *parent)
{
	TextContent *textContent = dynamic_cast<TextContent*>(itemContent);
	if (textContent)
		return new TextEditor(textContent, parent);

	HtmlContent *htmlContent = dynamic_cast<HtmlContent*>(itemContent);
	if (htmlContent)
		return new HtmlEditor(htmlContent, parent);

	ImageContent *imageContent = dynamic_cast<ImageContent*>(itemContent);
	if (imageContent)
		return new ImageEditor(imageContent, parent);

	AnimationContent *animationContent = dynamic_cast<AnimationContent*>(itemContent);
	if (animationContent)
		return new AnimationEditor(animationContent, parent);

	FileContent *fileContent = dynamic_cast<FileContent*>(itemContent); // Same for SoundContent
	if (fileContent)
		return new FileEditor(fileContent, parent);

	LinkContent *linkContent = dynamic_cast<LinkContent*>(itemContent);
	if (linkContent)
		return new LinkEditor(linkContent, parent);

	LauncherContent *launcherContent = dynamic_cast<LauncherContent*>(itemContent);
	if (launcherContent)
		return new LauncherEditor(launcherContent, parent);

	ColorContent *colorContent = dynamic_cast<ColorContent*>(itemContent);
	if (colorContent)
		return new ColorEditor(colorContent, parent);

	UnknownContent *unknownContent = dynamic_cast<UnknownContent*>(itemContent);
	if (unknownContent)
		return new UnknownEditor(unknownContent, parent);

	return 0;
}

void ItemEditor::setInlineEditor(QWidget *inlineEditor)
{
	m_widget   = inlineEditor;
	m_textEdit = 0;
	m_lineEdit = 0;

	KTextEdit *textEdit = dynamic_cast<KTextEdit*>(inlineEditor);
	if (textEdit)
		m_textEdit = textEdit;
	else {
		QLineEdit *lineEdit = dynamic_cast<QLineEdit*>(inlineEditor);
		if (lineEdit)
			m_lineEdit = lineEdit;
	}
}

#include <iostream>

/** Several possible scenarii to leave or not the edit mode:
  * - The user click on the basket, the edit widget then have a focusOut() event.
  *   The new focused widget (the basket) is not in the m_secondaryInlineWidgets list, so we leave the mode
  * - The user click on the font or color combobox of the rich text edit: the edit widget then have a focusOut() event.
  *   But the new focused widget is in the m_secondaryInlineWidgets list, so we do nothing.
  *   Then the user can reswitch focus to the edit widget or click the basket.
  *   The font or color combobox will then also send focusOut() event, this will be threated here like the event from the editbox.
  * - The user click another window from another application.
  *   kapp->focusWidget() and kapp->activeWindow() are both equal to 0 because out of our application.
  *   We close the editor. But only after a small laps of time: see below.
  * - The user click "Custom..." in the color combobox. A new KColorDialog is open.
  *   But kapp->focusWidget() and kapp->activeWindow() are both equal to 0 for a short laps of time.
  *   After the window is created and displayed, those values are then different from 0.
  *   To avoid to close the editor in this case (the application could believe the focus is now in another application, like in the previous case),
  *   we temporize and then re-test a second time: if the active window is a KColorDialog, it's OK.
  */
void ItemEditor::slotFocusOut()
{
	QValueList<QWidget*> inlineWidgets = m_secondaryInlineWidgets;
	inlineWidgets.append(widget());

	QWidget *focusWidget      = kapp->focusWidget();
	QObject *focusParent      = (focusWidget ? focusWidget->parent() : 0);
	QObject *focusGrandParent = (focusParent ? focusParent->parent() : 0);

//	std::cout << "------" << std::endl;
//	if (focusWidget)
//		std::cout << "FOCUS: " << focusWidget->className() << std::endl;
//	if (focusParent)
//		std::cout << "FOCUS PARENT: " << focusParent->className() << std::endl;
//	if (focusGrandParent)
//		std::cout << "FOCUS GRAND PARENT: " << focusGrandParent->className() << std::endl;

	for (QValueList<QWidget*>::iterator it = inlineWidgets.begin(); it != inlineWidgets.end(); ++it) {
//		std::cout << (*it)->className() << std::endl;
		// If current focused widget is this KTextEdit or the popup menu of the KTextEdit, do not consider that as a focus loss:
		if ( focusWidget == *it || (focusParent && focusParent == *it) || (focusGrandParent && focusGrandParent == *it) )
			return;
	}

	// When color dialog is called, focusWidget == 0L, but also when another application widget stealed focus. We need to differenciate those two cases:
	if (!focusWidget && !kapp->activeWindow())
		QTimer::singleShot(100, this, SLOT(slotFocusOut2()));
	else
		emit askValidation();
}

void ItemEditor::slotFocusOut2()
{
	if (!kapp->activeWindow())
		emit askValidation();
}

/** class TextEditor: */

TextEditor::TextEditor(TextContent *textContent, QWidget *parent)
 : ItemEditor(textContent), m_textContent(textContent)
{
	FocusedTextEdit *textEdit = new FocusedTextEdit(Settings::enterValidateInline(), /*disableUpdatesOnKeyPress=*/true, parent);
	textEdit->setTextFormat(Qt::PlainText);
	textEdit->setPaletteBackgroundColor(item()->backgroundColor());
	textEdit->setPaletteForegroundColor(item()->textColor());
	textEdit->setFont(item()->font());
	textEdit->setHScrollBarMode(QScrollView::AlwaysOff);
	if (Settings::spellCheckTextItems())
		textEdit->setCheckSpellingEnabled(true);
	textEdit->setText(m_textContent->text());
	textEdit->moveCursor(KTextEdit::MoveEnd, false); // FIXME: Sometimes, the cursor flicker at ends before being positionned where clicked (because kapp->processEvents() I think)
	setInlineEditor(textEdit);
	connect( textEdit, SIGNAL(escapePressed()), this, SIGNAL(askValidation()) );
	connect( textEdit, SIGNAL(focusOut()),      this, SLOT(slotFocusOut())    );
}

TextEditor::~TextEditor()
{
	delete widget(); // TODO: delete that in validate(), so we can remove one method
}

void TextEditor::validate()
{
	textEdit()->setCheckSpellingEnabled(false);
	if (textEdit()->text().isEmpty())
		setEmpty();
	m_textContent->setText(textEdit()->text());
	m_textContent->saveToFile();
	m_textContent->setEdited();

	if (Settings::spellCheckTextItems() != textEdit()->checkSpellingEnabled()) {
		Settings::setSpellCheckTextItems(textEdit()->checkSpellingEnabled());
		Settings::saveConfig();
	}

//	delete widget();
}

/** class HtmlEditor: */

HtmlEditor::HtmlEditor(HtmlContent *htmlContent, QWidget *parent)
 : ItemEditor(htmlContent), m_htmlContent(htmlContent)
{
	FocusedTextEdit *textEdit = new FocusedTextEdit(Settings::enterValidateInline(), /*disableUpdatesOnKeyPress=*/true, parent);
	textEdit->setTextFormat(Qt::RichText);
	textEdit->setPaletteBackgroundColor(item()->backgroundColor());
	textEdit->setPaletteForegroundColor(item()->textColor());
	textEdit->setFont(item()->font());
	textEdit->setHScrollBarMode(QScrollView::AlwaysOff);
	textEdit->setText(m_htmlContent->html());
	textEdit->moveCursor(KTextEdit::MoveEnd, false);
	setInlineEditor(textEdit);

	QValueList<QWidget*> widgets;
	widgets.append(InlineEditors::instance()->richTextFont);
	widgets.append(InlineEditors::instance()->richTextColor);
	widgets.append(InlineEditors::instance()->richTextToolBar());
	setSecondaryInlineWidgets(widgets);

	connect( textEdit,                                 SIGNAL(escapePressed()), this, SIGNAL(askValidation()) );
	connect( textEdit,                                 SIGNAL(focusOut()),      this, SLOT(slotFocusOut())    );
	connect( InlineEditors::instance()->richTextFont,  SIGNAL(focusOut()),      this, SLOT(slotFocusOut())    );
	connect( InlineEditors::instance()->richTextColor, SIGNAL(focusOut()),      this, SLOT(slotFocusOut())    );

	connect( InlineEditors::instance()->richTextFont,  SIGNAL(textChanged(const QString&)), textEdit, SLOT(setFamily(const QString&)) );
	connect( InlineEditors::instance()->richTextColor, SIGNAL(activated(const QColor&)),    textEdit, SLOT(setColor(const QColor&))   );

	connect( textEdit,  SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPositionChanged()) );
	connect( textEdit,  SIGNAL(clicked(int, int)),               this, SLOT(cursorPositionChanged()) );
//	connect( textEdit,  SIGNAL(currentVerticalAlignmentChanged(VerticalAlignment)), this, SLOT(slotVerticalAlignmentChanged()) );

	connect( InlineEditors::instance()->richTextBold,      SIGNAL(activated()), this, SLOT(setBold())      );
	connect( InlineEditors::instance()->richTextItalic,    SIGNAL(activated()), this, SLOT(setItalic())    );
	connect( InlineEditors::instance()->richTextUnderline, SIGNAL(activated()), this, SLOT(setUnderline()) );
	connect( InlineEditors::instance()->richTextLeft,      SIGNAL(activated()), this, SLOT(setLeft())      );
	connect( InlineEditors::instance()->richTextCenter,    SIGNAL(activated()), this, SLOT(setCentered())  );
	connect( InlineEditors::instance()->richTextRight,     SIGNAL(activated()), this, SLOT(setRight())     );
	connect( InlineEditors::instance()->richTextJustified, SIGNAL(activated()), this, SLOT(setBlock())     );

	InlineEditors::instance()->richTextToolBar()->show();
	cursorPositionChanged();
	InlineEditors::instance()->enableRichTextToolBar();
}

void HtmlEditor::cursorPositionChanged()
{
	InlineEditors::instance()->richTextFont->setCurrentFont(  textEdit()->currentFont().family() );
	if (InlineEditors::instance()->richTextColor->color() != textEdit()->color())
		InlineEditors::instance()->richTextColor->setColor(   textEdit()->color()                );
	InlineEditors::instance()->richTextBold->setChecked(      textEdit()->bold()                 );
	InlineEditors::instance()->richTextItalic->setChecked(    textEdit()->italic()               );
	InlineEditors::instance()->richTextUnderline->setChecked( textEdit()->underline()            );

//	std::cout << "Align: " << textEdit()->alignment() << std::endl;
	switch (textEdit()->alignment()) {
		default:
		case 1/*Qt::AlignLeft*/:     InlineEditors::instance()->richTextLeft->setChecked(true);      break;
		case 4/*Qt::AlignCenter*/:   InlineEditors::instance()->richTextCenter->setChecked(true);    break;
		case 2/*Qt::AlignRight*/:    InlineEditors::instance()->richTextRight->setChecked(true);     break;
		case -8/*Qt::AlignJustify*/: InlineEditors::instance()->richTextJustified->setChecked(true); break;
	}
}

/*void HtmlEditor::slotVe<rticalAlignmentChanged(QTextEdit::VerticalAlignment align)
{
	QTextEdit::VerticalAlignment align = textEdit()
	switch (align) {
		case KTextEdit::AlignSuperScript:
			InlineEditors::instance()->richTextSuper->setChecked(true);
			InlineEditors::instance()->richTextSub->setChecked(false);
			break;
		case KTextEdit::AlignSubScript:
			InlineEditors::instance()->richTextSuper->setChecked(false);
			InlineEditors::instance()->richTextSub->setChecked(true);
			break;
		default:
			InlineEditors::instance()->richTextSuper->setChecked(false);
			InlineEditors::instance()->richTextSub->setChecked(false);
	}

	ItemHtmlEditor::buttonToggled(int id) :
		case 106:
			if (isOn && m_toolbar->isButtonOn(107))
				m_toolbar->setButton(107, false);
			m_text->setVerticalAlignment(isOn ? KTextEdit::AlignSuperScript : KTextEdit::AlignNormal);
			break;
		case 107:
			if (isOn && m_toolbar->isButtonOn(106))
				m_toolbar->setButton(106, false);
			m_text->setVerticalAlignment(isOn ? KTextEdit::AlignSubScript   : KTextEdit::AlignNormal);
			break;
}*/

void HtmlEditor::setBold()      { textEdit()->setBold(      InlineEditors::instance()->richTextBold->isChecked()      ); }
void HtmlEditor::setItalic()    { textEdit()->setItalic(    InlineEditors::instance()->richTextItalic->isChecked()    ); }
void HtmlEditor::setUnderline() { textEdit()->setUnderline( InlineEditors::instance()->richTextUnderline->isChecked() ); }
void HtmlEditor::setLeft()      { textEdit()->setAlignment(Qt::AlignLeft);    }
void HtmlEditor::setCentered()  { textEdit()->setAlignment(Qt::AlignCenter);  }
void HtmlEditor::setRight()     { textEdit()->setAlignment(Qt::AlignRight);   }
void HtmlEditor::setBlock()     { textEdit()->setAlignment(Qt::AlignJustify); }

HtmlEditor::~HtmlEditor()
{
	delete widget();
}

void HtmlEditor::validate()
{
	if (Utilities::htmlToText(textEdit()->text()).isEmpty())
		setEmpty();
	m_htmlContent->setHtml(textEdit()->text());
	m_htmlContent->saveToFile();
	m_htmlContent->setEdited();

	disconnect();
	widget()->disconnect();
	InlineEditors::instance()->disableRichTextToolBar();

	InlineEditors::instance()->richTextToolBar()->hide();
	//	delete widget(); // FIXME: Crash when editing a second time and then acting on the toolbar (set bold...).
}

/** class ImageEditor: */

ImageEditor::ImageEditor(ImageContent *imageContent, QWidget *parent)
 : ItemEditor(imageContent)
{
	int choice = KMessageBox::questionYesNo(parent, i18n(
		"Images can not be edited here for the moment (the next version of BasKet Note Pads will include an image editor).\n"
		"Do you want to open it with an application that understand it?"),
		i18n("Edit Animation Item"),
		KStdGuiItem::open(),
		KStdGuiItem::cancel());

	if (choice == KMessageBox::Yes)
		item()->basket()->itemOpen(item());
}

/** class AnimationEditor: */

AnimationEditor::AnimationEditor(AnimationContent *animationContent, QWidget *parent)
 : ItemEditor(animationContent)
{
	int choice = KMessageBox::questionYesNo(parent, i18n(
		"This animated image can not be edited here.\n"
		"Do you want to open it with an application that understand it?"),
		i18n("Edit Animation Item"),
		KStdGuiItem::open(),
		KStdGuiItem::cancel());

	if (choice == KMessageBox::Yes)
		item()->basket()->itemOpen(item());
}

/** class FileEditor: */

FileEditor::FileEditor(FileContent *fileContent, QWidget *parent)
 : ItemEditor(fileContent), m_fileContent(fileContent)
{
	FocusedLineEdit *lineEdit = new FocusedLineEdit(/*escapeOnReturn=*/true, parent);
	lineEdit->setPaletteBackgroundColor(item()->backgroundColor());
	lineEdit->setPaletteForegroundColor(item()->textColor());
	lineEdit->setFont(item()->font());
	lineEdit->setText(m_fileContent->fileName());
	lineEdit->selectAll();
	setInlineEditor(lineEdit);
	connect( lineEdit, SIGNAL(escapePressed()), this, SIGNAL(askValidation()) );
	connect( lineEdit, SIGNAL(focusOut()),      this, SLOT(slotFocusOut())    );
}

FileEditor::~FileEditor()
{
	delete widget();
}

void FileEditor::validate()
{
	// FIXME: How to detect cancel?
	if (!lineEdit()->text().isEmpty() && m_fileContent->trySetFileName(lineEdit()->text())) {
		m_fileContent->setFileName(lineEdit()->text());
		m_fileContent->setEdited();
	}

	delete widget();
}

/** class LinkEditor: */

LinkEditor::LinkEditor(LinkContent *linkContent, QWidget *parent)
 : ItemEditor(linkContent)
{
	LinkEditDialog dialog(linkContent, parent);
	if (dialog.exec() == QDialog::Rejected)
		cancel();
	if (linkContent->url().isEmpty() && linkContent->title().isEmpty())
		setEmpty();
}

/** class LauncherEditor: */

LauncherEditor::LauncherEditor(LauncherContent *launcherContent, QWidget *parent)
 : ItemEditor(launcherContent)
{
	LauncherEditDialog dialog(launcherContent, parent);
	if (dialog.exec() == QDialog::Rejected)
		cancel();
	if (launcherContent->name().isEmpty() && launcherContent->exec().isEmpty())
		setEmpty();
}

/** class ColorEditor: */

ColorEditor::ColorEditor(ColorContent *colorContent, QWidget *parent)
 : ItemEditor(colorContent)
{
	KColorDialog dialog(parent, /*name=*/0, /*modal=*/true);
	dialog.setColor(colorContent->color());
	dialog.setCaption(i18n("Edit Color Item"));
	if (dialog.exec() == QDialog::Accepted) {
		if (dialog.color() != colorContent->color()) {
			colorContent->setColor(dialog.color());
			colorContent->setEdited();
		}
	} else
		cancel();

	/* This code don't allow to set a caption to the dialog:
	QColor color = colorContent()->color();
	if (KColorDialog::getColor(color, parent) == QDialog::Accepted && color != m_color) {
		colorContent()->setColor(color);
		setEdited();
	}*/
}

/** class UnknownEditor: */

UnknownEditor::UnknownEditor(UnknownContent *unknownContent, QWidget *parent)
 : ItemEditor(unknownContent)
{
	KMessageBox::information(parent, i18n(
		"The type of this item is unknown and can not be edited here.\n"
		"You however can drag or copy the item into an application that understand it."),
		i18n("Edit Unknown Item"));
}

/*********************************************************************/

/** class LinkEditDialog: */

LinkEditDialog::LinkEditDialog(LinkContent *contentItem, QWidget *parent/*, QKeyEvent *ke*/)
 : KDialogBase(KDialogBase::Plain, i18n("Edit Link Item"), KDialogBase::Ok | KDialogBase::Cancel,
               KDialogBase::Ok, parent, /*name=*/0, /*modal=*/true, /*separator=*/true),
   m_itemContent(contentItem)
{
	QWidget     *page   = plainPage();
	QGridLayout *layout = new QGridLayout(page, /*nRows=*/4, /*nCols=*/2, /*margin=*/0, spacingHint());

	m_url = new KURLRequester(m_itemContent->url().url(), page);

	QWidget *wid1 = new QWidget(page);
	QHBoxLayout *titleLay = new QHBoxLayout(wid1, /*margin=*/0, spacingHint());
	m_title = new QLineEdit(m_itemContent->title(), wid1);
	m_autoTitle = new QPushButton(i18n("Auto"), wid1);
	m_autoTitle->setToggleButton(true);
	m_autoTitle->setOn(m_itemContent->autoTitle());
	titleLay->addWidget(m_title);
	titleLay->addWidget(m_autoTitle);

	QWidget *wid = new QWidget(page);
	QHBoxLayout *hLay = new QHBoxLayout(wid, /*margin=*/0, spacingHint());
	m_icon = new KIconButton(wid);
	QLabel *label3 = new QLabel(m_icon, i18n("&Icon:"), page);
	KURL filteredURL = KURIFilter::self()->filteredURI(KURL(m_url->lineEdit()->text()));
	m_icon->setIconType(KIcon::NoGroup, KIcon::MimeType);
	m_icon->setIconSize(LinkLook::lookForURL(filteredURL)->iconSize());
	m_autoIcon = new QPushButton(i18n("Auto"), wid); // Create before to know size here:
	/* Icon button: */
	m_icon->setIcon(m_itemContent->icon());
	int minSize = m_autoIcon->sizeHint().height();
	// Make the icon button at least the same heigh than the other buttons for a better alignment (nicer to the eyes):
	if (m_icon->sizeHint().height() < minSize)
		m_icon->setFixedSize(minSize, minSize);
	else
		m_icon->setFixedSize(m_icon->sizeHint().height(), m_icon->sizeHint().height()); // Make it square
	/* Auto button: */
	m_autoIcon->setToggleButton(true);
	m_autoIcon->setOn(m_itemContent->autoIcon());
	hLay->addWidget(m_icon);
	hLay->addWidget(m_autoIcon);
	hLay->addStretch();

	m_url->lineEdit()->setMinimumWidth(m_url->lineEdit()->fontMetrics().maxWidth()*20);
	m_title->setMinimumWidth(m_title->fontMetrics().maxWidth()*20);

	//m_url->setShowLocalProtocol(true);
	QLabel      *label1 = new QLabel(m_url,   i18n("Ta&rget:"),    page);
	QLabel      *label2 = new QLabel(m_title, i18n("&Title:"),  page);
	layout->addWidget(label1,  0, 0, Qt::AlignVCenter);
	layout->addWidget(label2,  1, 0, Qt::AlignVCenter);
	layout->addWidget(label3,  2, 0, Qt::AlignVCenter);
	layout->addWidget(m_url,   0, 1, Qt::AlignVCenter);
	layout->addWidget(wid1,    1, 1, Qt::AlignVCenter);
	layout->addWidget(wid,     2, 1, Qt::AlignVCenter);

	m_isAutoModified = false;
	connect( m_url,   SIGNAL(textChanged(const QString&)), this, SLOT(urlChanged(const QString&))     );
	connect( m_title, SIGNAL(textChanged(const QString&)), this, SLOT(doNotAutoTitle(const QString&)) );
	connect( m_icon,  SIGNAL(iconChanged(QString))       , this, SLOT(doNotAutoIcon(QString))         );
	connect( m_autoTitle, SIGNAL(clicked()), this, SLOT(guessTitle()) );
	connect( m_autoIcon,  SIGNAL(clicked()), this, SLOT(guessIcon())  );

	QWidget *stretchWidget = new QWidget(page);
	stretchWidget->setSizePolicy(QSizePolicy(/*hor=*/QSizePolicy::Fixed, /*ver=*/QSizePolicy::Expanding, /*hStretch=*/1, /*vStretch=*/5000)); // Make it fill ALL vertical space
	layout->addWidget(stretchWidget, 3, 1, Qt::AlignVCenter);


	urlChanged("");

//	if (ke)
//		kapp->postEvent(m_url->lineEdit(), ke);
}

LinkEditDialog::~LinkEditDialog()
{
}

void LinkEditDialog::polish()
{
	KDialogBase::polish();
	if (m_url->lineEdit()->text().isEmpty()) {
		m_url->setFocus();
		m_url->lineEdit()->end(false);
	} else {
		m_title->setFocus();
		m_title->end(false);
	}
}


void LinkEditDialog::urlChanged(const QString&)
{
	m_isAutoModified = true;
//	guessTitle();
//	guessIcon();
	// Optimization (filter only once):
	KURL filteredURL = KURIFilter::self()->filteredURI(KURL(m_url->url()));
	if (m_autoIcon->isOn())
		m_icon->setIcon(ItemFactory::iconForURL(filteredURL));
	if (m_autoTitle->isOn()) {
		m_title->setText(ItemFactory::titleForURL(filteredURL));
		m_autoTitle->setOn(true); // Because the setText() will disable it!
	}
}

void LinkEditDialog::doNotAutoTitle(const QString&)
{
	if (m_isAutoModified)
		m_isAutoModified = false;
	else
		m_autoTitle->setOn(false);
}

void LinkEditDialog::doNotAutoIcon(QString)
{
	m_autoIcon->setOn(false);
}

void LinkEditDialog::guessIcon()
{
	if (m_autoIcon->isOn()) {
		KURL filteredURL = KURIFilter::self()->filteredURI(KURL(m_url->url()));
		m_icon->setIcon(ItemFactory::iconForURL(filteredURL));
	}
}

void LinkEditDialog::guessTitle()
{
	if (m_autoTitle->isOn()) {
		KURL filteredURL = KURIFilter::self()->filteredURI(KURL(m_url->url()));
		m_title->setText(ItemFactory::titleForURL(filteredURL));
		m_autoTitle->setOn(true); // Because the setText() will disable it!
	}
}

void LinkEditDialog::slotOk()
{
	KURL filteredURL = KURIFilter::self()->filteredURI(KURL(m_url->url()));
	m_itemContent->setLink(filteredURL, m_title->text(), m_icon->icon(), m_autoTitle->isOn(), m_autoIcon->isOn());
	m_itemContent->setEdited();

	/* Change icon size if link look have changed */
	LinkLook *linkLook = LinkLook::lookForURL(filteredURL);
	QString icon = m_icon->icon();             // When we change size, icon isn't changed and keep it's old size
	m_icon->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum); // Reset size policy
	m_icon->setIconSize(linkLook->iconSize()); //  So I store it's name and reload it after size change !
	m_icon->setIcon(icon);
	int minSize = m_autoIcon->sizeHint().height();
	// Make the icon button at least the same heigh than the other buttons for a better alignment (nicer to the eyes):
	if (m_icon->sizeHint().height() < minSize)
		m_icon->setFixedSize(minSize, minSize);
	else
		m_icon->setFixedSize(m_icon->sizeHint().height(), m_icon->sizeHint().height()); // Make it square

	KDialogBase::slotOk();
}

/** class LauncherEditDialog: */

LauncherEditDialog::LauncherEditDialog(LauncherContent *contentItem, QWidget *parent)
 : KDialogBase(KDialogBase::Plain, i18n("Edit Launcher Item"), KDialogBase::Ok | KDialogBase::Cancel,
               KDialogBase::Ok, parent, /*name=*/0, /*modal=*/true, /*separator=*/true),
   m_itemContent(contentItem)
{
	QWidget     *page   = plainPage();
	QGridLayout *layout = new QGridLayout(page, /*nRows=*/4, /*nCols=*/2, /*margin=*/0, spacingHint());

	KService service(contentItem->fullPath());

	m_command = new RunCommandRequester(service.exec(), i18n("Choose a command to run:"), page);
	m_name    = new QLineEdit(service.name(), page);

	QWidget *wid = new QWidget(page);
	QHBoxLayout *hLay = new QHBoxLayout(wid, /*margin=*/0, spacingHint());
	m_icon = new KIconButton(wid);
	QLabel *label = new QLabel(m_icon, i18n("&Icon:"), page);
	m_icon->setIconType(KIcon::NoGroup, KIcon::Application);
	m_icon->setIconSize(LinkLook::launcherLook->iconSize());
	QPushButton *guessButton = new QPushButton(i18n("&Guess"), wid);
	/* Icon button: */
	m_icon->setIcon(service.icon());
	int minSize = guessButton->sizeHint().height();
	// Make the icon button at least the same heigh than the other buttons for a better alignment (nicer to the eyes):
	if (m_icon->sizeHint().height() < minSize)
		m_icon->setFixedSize(minSize, minSize);
	else
		m_icon->setFixedSize(m_icon->sizeHint().height(), m_icon->sizeHint().height()); // Make it square
	/* Guess button: */
	hLay->addWidget(m_icon);
	hLay->addWidget(guessButton);
	hLay->addStretch();

	m_command->lineEdit()->setMinimumWidth(m_command->lineEdit()->fontMetrics().maxWidth()*20);

	QLabel *label1 = new QLabel(m_command->lineEdit(), i18n("Comman&d:"), page);
	QLabel *label2 = new QLabel(m_name,                i18n("&Name:"),    page);
	layout->addWidget(label1,    0, 0, Qt::AlignVCenter);
	layout->addWidget(label2,    1, 0, Qt::AlignVCenter);
	layout->addWidget(label,     2, 0, Qt::AlignVCenter);
	layout->addWidget(m_command, 0, 1, Qt::AlignVCenter);
	layout->addWidget(m_name,    1, 1, Qt::AlignVCenter);
	layout->addWidget(wid,       2, 1, Qt::AlignVCenter);

	QWidget *stretchWidget = new QWidget(page);
	stretchWidget->setSizePolicy(QSizePolicy(/*hor=*/QSizePolicy::Fixed, /*ver=*/QSizePolicy::Expanding, /*hStretch=*/1, /*vStretch=*/5000)); // Make it fill ALL vertical space
	layout->addWidget(stretchWidget, 3, 1, Qt::AlignVCenter);

	connect( guessButton, SIGNAL(clicked()), this, SLOT(guessIcon()) );
}

LauncherEditDialog::~LauncherEditDialog()
{
}

void LauncherEditDialog::polish()
{
	KDialogBase::polish();
	if (m_command->runCommand().isEmpty()) {
		m_command->lineEdit()->setFocus();
		m_command->lineEdit()->end(false);
	} else {
		m_name->setFocus();
		m_name->end(false);
	}
}

void LauncherEditDialog::slotOk()
{
	// TODO: Remember if a string has been modified AND IS DIFFERENT FROM THE ORIGINAL!

	KConfig conf(m_itemContent->fullPath());
	conf.setGroup("Desktop Entry");
	conf.writeEntry("Exec", m_command->runCommand());
	conf.writeEntry("Name", m_name->text());
	conf.writeEntry("Icon", m_icon->icon());

	// Just for faster feedback: conf object will save to disk (and then m_item->loadContent() called)
	m_itemContent->setLauncher(m_name->text(), m_icon->icon(), m_command->runCommand());
	m_itemContent->setEdited();

	KDialogBase::slotOk();
}

void LauncherEditDialog::guessIcon()
{
	m_icon->setIcon( ItemFactory::iconForCommand(m_command->runCommand()) );
}

/** class InlineEditors: */

InlineEditors::InlineEditors()
{
	m_mainWindow = 0;
}

InlineEditors::~InlineEditors()
{
}

InlineEditors* InlineEditors::instance()
{
	static InlineEditors *instance = 0;
	if (!instance)
		instance = new InlineEditors();
	return instance;
}

void InlineEditors::initToolBars(KMainWindow *mainWindow)
{
	m_mainWindow = mainWindow;

	// Init the RichTextEditor Toolbar:

	richTextFont = new FocusedFontCombo(mainWindow);
	KWidgetAction *action = new KWidgetAction(richTextFont, i18n("Font"), Qt::Key_F6,
	                                          /*receiver=*/0, /*slot=*/"", mainWindow->actionCollection(), "richtext_font");
	richTextColor = new FocusedColorCombo(mainWindow);
	action = new KWidgetAction(richTextColor, i18n("Color"), KShortcut(), 0, SLOT(), mainWindow->actionCollection(), "richtext_color");

	richTextBold      = new KToggleAction( i18n("Bold"),        "text_bold",   "Ctrl+B", mainWindow->actionCollection(), "richtext_bold"      );
	richTextItalic    = new KToggleAction( i18n("Italic"),      "text_italic", "Ctrl+I", mainWindow->actionCollection(), "richtext_italic"    );
	richTextUnderline = new KToggleAction( i18n("Underline"),   "text_under",  "Ctrl+U", mainWindow->actionCollection(), "richtext_underline" );

//	richTextSuper     = new KToggleAction( i18n("Superscript"), "text_super",  "",       mainWindow->actionCollection(), "richtext_super"     );
//	richTextSub       = new KToggleAction( i18n("Subscript"),   "text_sub",    "",       mainWindow->actionCollection(), "richtext_sub"       );

	richTextLeft      = new KToggleAction( i18n("Left Align"),  "text_left",   "",       mainWindow->actionCollection(), "richtext_left"      );
	richTextCenter    = new KToggleAction( i18n("Centered"),    "text_center", "",       mainWindow->actionCollection(), "richtext_center"    );
	richTextRight     = new KToggleAction( i18n("Right Align"), "text_right",  "",       mainWindow->actionCollection(), "richtext_right"     );
	richTextJustified = new KToggleAction( i18n("Justified"),   "text_block",  "",       mainWindow->actionCollection(), "richtext_block"     );

	richTextLeft->setExclusiveGroup("rt_justify");
	richTextCenter->setExclusiveGroup("rt_justify");
	richTextRight->setExclusiveGroup("rt_justify");
	richTextJustified->setExclusiveGroup("rt_justify");

	disableRichTextToolBar();
}

KToolBar* InlineEditors::richTextToolBar()
{
	if (m_mainWindow) {
		m_mainWindow->toolBar(); // Make sure we create the main toolbar FIRST, so it will be on top of the edit toolbar!
		return m_mainWindow->toolBar("richTextEditToolBar");
	} else
		return 0;
}

void InlineEditors::enableRichTextToolBar()
{
	richTextFont->setEnabled(true);
	richTextColor->setEnabled(true);
	richTextBold->setEnabled(true);
	richTextItalic->setEnabled(true);
	richTextUnderline->setEnabled(true);
	richTextLeft->setEnabled(true);
	richTextCenter->setEnabled(true);
	richTextRight->setEnabled(true);
	richTextJustified->setEnabled(true);
}

void InlineEditors::disableRichTextToolBar()
{
	disconnect(richTextFont);
	disconnect(richTextColor);
	disconnect(richTextBold);
	disconnect(richTextItalic);
	disconnect(richTextUnderline);
	disconnect(richTextLeft);
	disconnect(richTextCenter);
	disconnect(richTextRight);
	disconnect(richTextJustified);

	richTextFont->setEnabled(false);
	richTextColor->setEnabled(false);
	richTextBold->setEnabled(false);
	richTextItalic->setEnabled(false);
	richTextUnderline->setEnabled(false);
	richTextLeft->setEnabled(false);
	richTextCenter->setEnabled(false);
	richTextRight->setEnabled(false);
	richTextJustified->setEnabled(false);
}

#include "itemedit.moc"
