/***************************************************************************
 *   Copyright (C) 2003 by Sbastien Laot                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ITEMEDIT_H
#define ITEMEDIT_H

#include <kdialogbase.h>
#include <qtextedit.h>

class QWidget;
class QLineEdit;
class QPushButton;
class KIconButton;
class KURLRequester;
class KTextEdit;
class KMainWindow;
class KTooolBar;
class KToggleAction;

class Item;
class RunCommandRequester;
class FocusedFontCombo;
class FocusedColorCombo;

#include "itemcontent.h"

/** The base class for every item editors.
  * Scenario:
  *  The Basket class calls ItemEditor::editItemContent() with the ItemContent to edit.
  *  This method create the good child ItemEditor depending
  *  on the item content type and return it to the Basket.
  *  This custom ItemEditor have two choices regarding what to do in its constructor:
  *   - Display a dialog and then call cancel() if the user canceled the dialog;
  *   - Create an inline editor and call setInlineEditor() with that editor as parameter.
  *     When the user exit the edition, validate() is called by the Basket.
  *     You should then call setEmpty() is the user cleared the content.
  *  The custom editor SHOULD call the ItemEditor constructor.
  *  If the user cleared the content OR if the user canceled the dialog whereas he/she
  *  JUST ADDED the item, then the item will be deleted by the Basket.
  */
class ItemEditor : public QObject
{
  Q_OBJECT
  public:
	ItemEditor(ItemContent *itemContent);
	bool        isEmpty()  { return m_isEmpty;      }
	bool        canceled() { return m_canceled;     }
	bool        isInline() { return m_widget != 0;  }
	QWidget*    widget()   { return m_widget;       }
	KTextEdit*  textEdit() { return m_textEdit;     }
	QLineEdit*  lineEdit() { return m_lineEdit;     }

  private:
	bool         m_isEmpty;
	bool         m_canceled;
	QWidget     *m_widget;
	KTextEdit   *m_textEdit;
	QLineEdit   *m_lineEdit;
	ItemContent *m_itemContent;

  public:
	ItemContent* itemContent() { return m_itemContent; }
	Item*        item();
  protected:
	void setEmpty() { m_isEmpty  = true; }
	void cancel()   { m_canceled = true; }
	void setInlineEditor(QWidget *inlineEditor);
  public:
	virtual void validate() {}

  protected slots:
	void slotFocusOut();
	void slotFocusOut2();
  protected:
	void setSecondaryInlineWidgets(QValueList<QWidget*> widgets) { m_secondaryInlineWidgets = widgets; }
  private:
	QValueList<QWidget*> m_secondaryInlineWidgets;

  signals:
	void askValidation();

  public:
	static ItemEditor* editItemContent(ItemContent *itemContent, QWidget *parent);
};

class TextEditor : public ItemEditor
{
  Q_OBJECT
  public:
	TextEditor(TextContent *textContent, QWidget *parent);
	~TextEditor();
	void validate();
  protected:
	TextContent *m_textContent;
};

class HtmlEditor : public ItemEditor
{
  Q_OBJECT
  public:
	HtmlEditor(HtmlContent *htmlContent, QWidget *parent);
	~HtmlEditor();
	void validate();
  protected:
	HtmlContent *m_htmlContent;
  public slots:
	void cursorPositionChanged();
  protected slots:
//	void slotVerticalAlignmentChanged(QTextEdit::VerticalAlignment align);
	void setBold();
	void setItalic();
	void setUnderline();
	void setLeft();
	void setCentered();
	void setRight();
	void setBlock();
};

class ImageEditor : public ItemEditor
{
  Q_OBJECT
  public:
	ImageEditor(ImageContent *imageContent, QWidget *parent);
};

class AnimationEditor : public ItemEditor
{
  Q_OBJECT
  public:
	AnimationEditor(AnimationContent *animationContent, QWidget *parent);
};

class FileEditor : public ItemEditor
{
  Q_OBJECT
  public:
	FileEditor(FileContent *fileContent, QWidget *parent);
	~FileEditor();
	void validate();
  protected:
	FileContent *m_fileContent;
};

class LinkEditor : public ItemEditor
{
  Q_OBJECT
  public:
	LinkEditor(LinkContent *linkContent, QWidget *parent);
};

class LauncherEditor : public ItemEditor
{
  Q_OBJECT
  public:
	LauncherEditor(LauncherContent *launcherContent, QWidget *parent);
};

class ColorEditor : public ItemEditor
{
  Q_OBJECT
  public:
	ColorEditor(ColorContent *colorContent, QWidget *parent);
};

class UnknownEditor : public ItemEditor
{
  Q_OBJECT
  public:
	  UnknownEditor(UnknownContent *unknownContent, QWidget *parent);
};

/** The dialog to edit Link Item content.
  * @author Sbastien Laot
  */
class LinkEditDialog : public KDialogBase
{
  Q_OBJECT
  public:
	LinkEditDialog(LinkContent *contentItem, QWidget *parent = 0);
	~LinkEditDialog();
	void polish();
  protected slots:
	void slotOk();
	void urlChanged(const QString&);
	void doNotAutoTitle(const QString&);
	void doNotAutoIcon(QString);
	void guessTitle();
	void guessIcon();
  private:
	LinkContent   *m_itemContent;
	bool           m_isAutoModified;
	KURLRequester *m_url;
	QLineEdit     *m_title;
	KIconButton   *m_icon;
	QPushButton   *m_autoTitle;
	QPushButton   *m_autoIcon;
};


/** The dialog to edit Launcher Item content.
  * @author Sbastien Laot
  */
class LauncherEditDialog : public KDialogBase
{
  Q_OBJECT
  public:
	LauncherEditDialog(LauncherContent *contentItem, QWidget *parent = 0);
	~LauncherEditDialog();
	void polish();
  protected slots:
	void slotOk();
	void guessIcon();
  private:
	LauncherContent     *m_itemContent;
	RunCommandRequester *m_command;
	QLineEdit           *m_name;
	KIconButton         *m_icon;
};

/** This class manage toolbars for the inline editors.
  * The toolbars should be created once at the application startup,
  * then KXMLGUI can manage them and save theire state and position...
  * @author Sbastien Laot
  */
class InlineEditors : public QObject
{
  Q_OBJECT
  public:
	InlineEditors();
	~InlineEditors();
	void initToolBars(KMainWindow *mainWindow);
	static InlineEditors* instance();
  private:
	KMainWindow *m_mainWindow;
  public:
	// Rich Text ToolBar:
	KToolBar* richTextToolBar();
	void enableRichTextToolBar();
	void disableRichTextToolBar();
	FocusedFontCombo  *richTextFont;
	FocusedColorCombo *richTextColor;
	KToggleAction     *richTextBold;
	KToggleAction     *richTextItalic;
	KToggleAction     *richTextUnderline;
//	KToggleAction     *richTextSuper;
//	KToggleAction     *richTextSub;
	KToggleAction     *richTextLeft;
	KToggleAction     *richTextCenter;
	KToggleAction     *richTextRight;
	KToggleAction     *richTextJustified;
};

#endif // ITEMEDIT_H
